/*
** time.c - Utility functions for handling variables of type "timeval"
**
**          Used by the example applications.
*/

#include <stdio.h>
#include <stdlib.h>

#include "common.h"

struct timeval
get_time(void)
{
  struct timeval t;
  int     err;

  err = gettimeofday(&t, NULL);
  if (err) {
    printf("*** get_time: gettimeofday failed\n");
    exit(-1);
  }

  return t;
}

double
time_to_double(
  const struct timeval* t1)
{
  return
    ((double) t1->tv_usec) / 1000000.0 +
    ((double) t1->tv_sec);
}

struct timeval
double_to_time(
  double t1)
{
  struct timeval t;

  t.tv_sec = (long) t1;
  t.tv_usec = (long) ((t1 - (double) (t.tv_sec)) * 1000000.0);

  return t;
}

int
time_after(
  const struct timeval* t1,
  const struct timeval* t2)
{
  return (t1->tv_sec > t2->tv_sec || t1->tv_usec > t2->tv_usec);
}

int
time_equal(
  const struct timeval* t1,
  const struct timeval* t2)
{
  return (t1->tv_sec == t2->tv_sec && t1->tv_usec == t2->tv_usec);
}

int
time_before(
  const struct timeval* t1,
  const struct timeval* t2)
{
  return (t1->tv_sec < t2->tv_sec || t1->tv_usec < t2->tv_usec);
}

struct timeval
time_add(
  const struct timeval* t1,
  const struct timeval* t2)
{
  struct timeval t;

  t.tv_usec = t1->tv_usec + t2->tv_usec;
  t.tv_sec = t1->tv_sec + t2->tv_sec;
  if (t.tv_usec >= 1000000) {
    t.tv_usec -= 1000000;
    t.tv_sec++;
  }

  return t;
}

struct timeval
time_subtract(
  const struct timeval* t1,
  const struct timeval* t2)
{
  struct timeval t;

  t.tv_usec = t1->tv_usec - t2->tv_usec;
  t.tv_sec = t1->tv_sec - t2->tv_sec;
  if (t.tv_usec < 0) {
    t.tv_usec += 1000000;
    t.tv_sec--;
  }

  return t;
}

